# -*- coding: utf-8 -*-

__all__ = [
    'IActivitiesPersistence',
    'ActivitiesMemoryPersistence',
    'ActivitiesMongoDbPersistence'
]

from .IActivitiesPersistence import IActivitiesPersistence
from .ActivitiesMemoryPersistence import ActivitiesMemoryPersistence
from .ActivitiesMongoDbPersistence import ActivitiesMongoDbPersistence


# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Optional

from pip_services4_components.context import IContext
from pip_services4_data.query import DataPage, FilterParams, PagingParams

from ..data import AccountV1


class IAccountsPersistence(ABC):
    @abstractmethod
    def get_page_by_filter(self, context: Optional[IContext], filter: FilterParams, paging: PagingParams) -> DataPage:
        pass

    @abstractmethod
    def get_one_by_id(self, context: Optional[IContext], id: str) -> Optional[AccountV1]:
        pass

    @abstractmethod
    def get_one_by_login(self, context: Optional[IContext], login: str) -> Optional[AccountV1]:
        pass

    @abstractmethod
    def get_one_by_id_or_login(self, context: Optional[IContext], id_or_login: str) -> Optional[AccountV1]:
        pass

    @abstractmethod
    def create(self, context: Optional[IContext], account: AccountV1) -> AccountV1:
        pass

    @abstractmethod
    def update(self, context: Optional[IContext], account: AccountV1) -> AccountV1:
        pass

    @abstractmethod
    def delete_by_id(self, context: Optional[IContext], id: str) -> Optional[AccountV1]:
        pass



# -*- coding: utf-8 -*-

__all__ = ['ReferenceV1', 'ReferenceV1Schema', 'PartyActivityV1', 'PartyActivityV1Schema']

from .ReferenceV1 import ReferenceV1
from .ReferenceV1Schema import ReferenceV1Schema
from .PartyActivityV1 import PartyActivityV1
from .PartyActivityV1Schema import PartyActivityV1Schema 


# -*- coding: utf-8 -*-

__all__ = [
    'ISettingsPersistence', 'SettingsMemoryPersistence', 'SettingsMongoDbPersistence'
]

from .ISettingsPersistence import ISettingsPersistence
from .SettingsMemoryPersistence import SettingsMemoryPersistence
from .SettingsMongoDbPersistence import SettingsMongoDbPersistence



# -*- coding: utf-8 -*-
from typing import Any, Optional
from datetime import datetime

from pip_services4_data.data import IStringIdentifiable
from pip_services4_data.keys import IdGenerator


class AccountV1(IStringIdentifiable):
    def __init__(
        self,
        id: Optional[str] = None,
        login: Optional[str] = None,
        name: Optional[str] = None
    ):
        self.id: str = id or IdGenerator.next_long()
        self.login: Optional[str] = login
        self.name: Optional[str] = name

        # Activity tracking
        self.create_time: datetime = datetime.utcnow()
        self.deleted: Optional[bool] = False
        self.active: bool = True

        # User preferences
        self.about: Optional[str] = None
        self.time_zone: Optional[str] = None
        self.language: Optional[str] = None
        self.theme: Optional[str] = None

        # Custom fields
        self.custom_hdr: Optional[Any] = None
        self.custom_dat: Optional[Any] = None

    def to_dict(self):
        return self.__dict__



# -*- coding: utf-8 -*-
from pip_services4_commons.convert import TypeCode
from pip_services4_data.validate import ObjectSchema


class ReferenceV1Schema(ObjectSchema):
    """
    Schema for validating ReferenceV1 objects
    """
    
    def __init__(self):
        super().__init__()
        
        self.with_required_property('id', TypeCode.String)
        self.with_required_property('type', TypeCode.String)
        self.with_optional_property('name', TypeCode.String) 


# -*- coding: utf-8 -*-
import os
import pytest
from pip_services4_components.config import ConfigParams

from eic_aichat_users.accounts.persistence.AccountsMongoDbPersistence import AccountsMongoDbPersistence
from test.accounts.persistence.AccountsPersistenceFixture import AccountsPersistenceFixture


mongodb_uri = os.environ.get('MONGO_SERVICE_URI')
mongodb_host = os.environ.get('MONGO_SERVICE_HOST') or 'localhost'
mongodb_port = os.environ.get('MONGO_SERVICE_PORT') or 27017
mongodb_database = os.environ.get('MONGO_SERVICE_DB') or 'test'


@pytest.mark.skipif(not mongodb_uri and not mongodb_host, reason="MongoDB connection is not set")
class TestAccountsMongoDbPersistence:
    persistence: AccountsMongoDbPersistence
    fixture: AccountsPersistenceFixture

    def setup_method(self):
        self.persistence = AccountsMongoDbPersistence()
        self.persistence.configure(ConfigParams.from_tuples(
            'connection.uri', mongodb_uri,
            'connection.host', mongodb_host,
            'connection.port', mongodb_port,
            'connection.database', mongodb_database,
        ))

        self.fixture = AccountsPersistenceFixture(self.persistence)

        self.persistence.open(None)
        self.persistence.clear(None)

    def teardown_method(self):
        self.persistence.close(None)

    def test_crud_operations(self):
        self.fixture.test_crud_operations()

    def test_get_with_filter(self):
        self.fixture.test_get_with_filter()



# -*- coding: utf-8 -*-

__all__ = ['IPasswordsService', 'PasswordsService', 'ActivitiesConnector']

from .IPasswordsService import IPasswordsService
from .PasswordsService import PasswordsService
from .ActivitiesConnector import ActivitiesConnector



# sessions_service.py
from typing import Optional, Any
from datetime import datetime, timedelta

from pip_services4_components.config import ConfigParams, IConfigurable
from pip_services4_components.context import IContext
from pip_services4_components.refer import IReferenceable, IReferences, Descriptor
from pip_services4_data.query import FilterParams, PagingParams, DataPage
from pip_services4_observability.log import CompositeLogger

from ..data import SessionV1
from ..persistence.ISessionsPersistence import ISessionsPersistence

class SessionsService(IConfigurable, IReferenceable):
    def __init__(self):
        self._logger: CompositeLogger = CompositeLogger()
        self._persistence: Optional[ISessionsPersistence] = None
        self._expire_timeout = 24 * 3600  # default 24 hours in seconds

    def configure(self, config: ConfigParams):
        self._expire_timeout = config.get_as_integer_with_default("options.expire_timeout", self._expire_timeout)

    def set_references(self, references: IReferences):
        self._logger.set_references(references)
        self._persistence = references.get_one_required(
            Descriptor('aichatusers-sessions', 'persistence', '*', '*', '1.0')
        )

    def get_sessions(self, context: Optional[IContext], filter_params: FilterParams, paging: PagingParams) -> DataPage:
        return self._persistence.get_page_by_filter(context, filter_params, paging)

    def get_session_by_id(self, context: Optional[IContext], session_id: str) -> Optional[SessionV1]:
        return self._persistence.get_one_by_id(context, session_id)

    def open_session(self, context: Optional[IContext], user_id: str, user_name: str,
                     address: str, client: str, user: Any, data: Any) -> SessionV1:
        session = SessionV1(user_id=user_id, user_name=user_name, address=address, client=client)
        session.user = user
        session.data = data
        return self._persistence.create(context, session)

    def store_session_data(self, context: Optional[IContext], session_id: str, data: Any) -> Optional[SessionV1]:
        session = self._persistence.get_one_by_id(context, session_id)
        if session:
            session.data = data
            session.request_time = datetime.utcnow()
            return self._persistence.update(context, session)
        return None

    def update_session_user(self, context: Optional[IContext], session_id: str, user: Any) -> Optional[SessionV1]:
        session = self._persistence.get_one_by_id(context, session_id)
        if session:
            session.user = user
            session.request_time = datetime.utcnow()
            return self._persistence.update(context, session)
        return None

    def close_session(self, context: Optional[IContext], session_id: str) -> Optional[SessionV1]:
        session = self._persistence.get_one_by_id(context, session_id)
        if session:
            session.active = False
            session.close_time = datetime.utcnow()
            session.request_time = session.close_time
            session.user = None
            session.data = None
            return self._persistence.update(context, session)
        return None

    def close_expired_sessions(self, context: Optional[IContext]):
        request_time = datetime.utcnow() - timedelta(seconds=self._expire_timeout)
        self._persistence.close_expired(context, request_time)

    def delete_session_by_id(self, context: Optional[IContext], session_id: str) -> Optional[SessionV1]:
        return self._persistence.delete_by_id(context, session_id)



# -*- coding: utf-8 -*-
from typing import Any, Optional
from datetime import datetime

from pip_services4_data.data import IStringIdentifiable
from pip_services4_data.keys import IdGenerator


class UserPasswordV1(IStringIdentifiable):
    def __init__(
            self,
            id: Optional[str] = None,
            password: Optional[str] = None,
    ):
        # Identification 
        self.id: str = id or IdGenerator.next_long()
        self.password: str = password

        # Password management
        self.change_time: Optional[datetime] = None
        self.locked: bool = False
        self.lock_time: Optional[datetime] = None
        self.fail_count: int = 0
        self.fail_time: Optional[datetime] = None
        self.rec_code: Optional[str] = None
        self.rec_expire_time: Optional[datetime] = None

        # Custom fields
        self.custom_hdr: Optional[Any] = None
        self.custom_dat: Optional[Any] = None

    def to_dict(self):
        return self.__dict__



# -*- coding: utf-8 -*-
from pip_services4_persistence.persistence import IdentifiableMemoryPersistence

from .IPasswordsPersistence import IPasswordsPersistence


class PasswordsMemoryPersistence(IdentifiableMemoryPersistence, IPasswordsPersistence):

    def __init__(self):
        super().__init__()


# -*- coding: utf-8 -*-
from datetime import datetime, timedelta
import hashlib
import random
import string
from typing import Optional

from pip_services4_components.config import ConfigParams, IConfigurable
from pip_services4_components.refer import IReferenceable, IReferences, Descriptor
from pip_services4_observability.log import CompositeLogger
from pip_services4_data.keys import IdGenerator
from pip_services4_components.context import IContext
from pip_services4_commons.errors import BadRequestException, NotFoundException

from .ActivitiesConnector import ActivitiesConnector
from ...activities.logic import ActivitiesService

from ..persistence.IPasswordsPersistence import IPasswordsPersistence

from ..data.version1.UserPasswordV1 import UserPasswordV1
from ..data.version1.UserPasswordInfoV1 import UserPasswordInfoV1

from .IPasswordsService import IPasswordsService

# TODO
class DummyMessageDistributionClient:
    def sendPasswordChangedEmail(self, context, user_id):
        pass

    def sendAccountLockedEmail(self, context, user_id):
        pass

    def sendRecoverPasswordEmail(self, context, user_id):
        pass

class PasswordsService(IConfigurable, IReferenceable, IPasswordsService):
    def __init__(self):
        self._persistence: IPasswordsPersistence = None
        self._logger: CompositeLogger = CompositeLogger()
        self._activities_connector: ActivitiesConnector = None
        self._activities_service: ActivitiesService = None
        self._message_distribution_client = DummyMessageDistributionClient()

        self._lock_timeout = 1800000 # 30 mins
        self._attempt_timeout = 60000 # 1 min
        self._attempt_count = 4 # 4 times
        self._rec_expire_timeout = 24 * 3600000 # 24 hours
        self._lock_enabled = False
        self._magic_code = None
        self._code_length = 9 # Generated code length

        self._max_password_len = 20
        self._min_password_len = 5

        self._old_passwords_check = False
        self._old_passwords_count = 6


    def configure(self, config: ConfigParams):
        self._lock_timeout = config.get_as_integer_with_default('options.lock_timeout', self._lock_timeout)
        self._attempt_timeout = config.get_as_integer_with_default('options.attempt_timeout', self._attempt_timeout)
        self._attempt_count = config.get_as_integer_with_default('options.attempt_count', self._attempt_count)
        self._rec_expire_timeout = config.get_as_integer_with_default('options.rec_expire_timeout', self._rec_expire_timeout)
        self._lock_enabled = config.get_as_boolean_with_default('options.lock_enabled', self._lock_enabled)
        self._magic_code = config.get_as_string_with_default('options.magic_code', self._magic_code)
        self._code_length = max(3, min(9, config.get_as_integer_with_default('options.code_length', self._code_length)))

        self._max_password_len = config.get_as_integer_with_default('options.max_password_len', self._max_password_len)
        self._min_password_len = config.get_as_integer_with_default('options.min_password_len', self._min_password_len)

        self._old_passwords_check = config.get_as_boolean_with_default('options.old_passwords_check', self._old_passwords_check)
        self._old_passwords_count = config.get_as_integer_with_default('options.old_passwords_count', self._old_passwords_count)

    def set_references(self, references: IReferences):
        self._logger.set_references(references)
        self._persistence = references.get_one_required(Descriptor('aichatusers-passwords', 'persistence', '*', '*', '1.0'))
        
        self._activities_service = references.get_one_optional(Descriptor('aichatusers-activities', 'service', '*', '*', '1.0'))
        if self._activities_service is not None:
            self._activities_service.set_references(references)
        
        self._activities_connector = ActivitiesConnector(self._logger, self._activities_service)

    def _generate_verification_code(self) -> str:
        return IdGenerator.next_short()[:self._code_length]
    
    def _hash_password(self, password: str) -> str:
        if not password:
            return None
        return hashlib.sha256(password.encode()).hexdigest()
    
    def _add_old_password(self, password_obj: UserPasswordV1, old_password: str) -> UserPasswordV1:
        if not password_obj.custom_dat:
            password_obj.custom_dat = {'old_passwords': []}
        old_pwds = password_obj.custom_dat.get('old_passwords', [])
        if len(old_pwds) >= self._old_passwords_count:
            old_pwds = old_pwds[1:]
        old_pwds.append(old_password)
        password_obj.custom_dat['old_passwords'] = old_pwds
        return password_obj
    
    def _generate_random_password(self, length: Optional[int] = None) -> str:
        length = length or random.randint(12, 16)
        return ''.join(random.choices(string.printable[33:127], k=length))
    
    def _verify_password(self, context: Optional[IContext], password: str, user_id: Optional[str] = None) -> None:
        if not password:
            raise BadRequestException(
                context,
                'NO_PASSWORD',
                'Missing user password'
            )

        if len(password) < self._min_password_len or len(password) > self._max_password_len:
            raise BadRequestException(
                context,
                'BAD_PASSWORD',
                f'User password should be {self._min_password_len} to {self._max_password_len} symbols long'
            )

        if user_id and self._old_passwords_check:
            old_password_err = None
            try:
                user_password = self._read_user_password(context, user_id)
            except NotFoundException:
                return
            except Exception as err:
                raise err

            if user_password:
                if not user_password.custom_dat:
                    user_password.custom_dat = {'old_passwords': []}
                old_passwords = user_password.custom_dat.get('old_passwords', [])

                hashed = self._hash_password(password)
                for old_pass in old_passwords:
                    if old_pass == hashed:
                        old_password_err = BadRequestException(
                            context,
                            'OLD_PASSWORD',
                            'Old password used'
                        )

            if old_password_err:
                raise old_password_err
            
    def validate_password(self, context: Optional[IContext], password: str) -> None:
        self._verify_password(context, password, None)

    def _read_user_password(self, context: Optional[IContext], user_id: str) -> UserPasswordV1:
        item = self._persistence.get_one_by_id(context, user_id)

        if item is None:
            raise NotFoundException(
                context,
                'USER_NOT_FOUND',
                f'User {user_id} was not found'
            ).with_details('user_id', user_id)

        return item

    def validate_password_for_user(self, context: Optional[IContext], user_id: str, password: str) -> None:
        self._verify_password(context, password, user_id)

    def get_password_info(self, context: Optional[IContext], user_id: str) -> UserPasswordInfoV1:
        data = self._persistence.get_one_by_id(context, user_id)

        if data is not None:
            return UserPasswordInfoV1(
                id=data.id,
                change_time=data.change_time,
                locked=data.locked,
                lock_time=data.lock_time
            )
        return None

    def set_password(self, context: Optional[IContext], user_id: str, password: str) -> None:
        self._verify_password(context, password, user_id)

        userPassword = self._persistence.get_one_by_id(context, user_id)
        password = self._hash_password(password)

        if userPassword is not None:
            userPassword = self._add_old_password(userPassword, userPassword.password)
        else:
            userPassword = UserPasswordV1(user_id, password)

        self._persistence.create(context, userPassword)

    def set_temp_password(self, context: Optional[IContext], user_id: str) -> str:
        password = self._generate_random_password()
        passwordHash = self._hash_password(password)

        userPassword = UserPasswordV1(user_id, passwordHash)
        userPassword.change_time = datetime.now()

        self._persistence.create(context, userPassword)
        return password

    def delete_password(self, context: Optional[IContext], user_id: str) -> None:
        self._persistence.delete_by_id(context, user_id)

    def authenticate(self, context: Optional[IContext], user_id: str, password: str) -> bool:
        hashed_password = self._hash_password(password)
        current_time = datetime.utcnow()

        user_password = self._read_user_password(context, user_id)

        password_match = user_password.password == hashed_password
        last_failure_timeout = (
            (current_time - user_password.fail_time).total_seconds() * 1000
            if user_password.fail_time else None
        )

        if not self._lock_enabled and password_match:
            user_password.locked = False
        else:
            if password_match and user_password.locked and last_failure_timeout and last_failure_timeout > self._lock_timeout:
                user_password.locked = False
            elif user_password.locked:
                raise BadRequestException(
                    context,
                    'ACCOUNT_LOCKED',
                    f'Account for user {user_id} is locked'
                ).with_details('user_id', user_id)

            if not password_match:
                if last_failure_timeout is None or last_failure_timeout < self._attempt_timeout:
                    user_password.fail_count = (user_password.fail_count or 0) + 1

                user_password.fail_time = current_time

                try:
                    self._persistence.update(context, user_password)
                except Exception as err:
                    self._logger.error(context, err, 'Failed to save user password')

                if user_password.fail_count >= self._attempt_count:
                    user_password.locked = True

                    self._message_distribution_client.sendAccountLockedEmail(context, user_id)

                    raise BadRequestException(
                        context,
                        'ACCOUNT_LOCKED',
                        f'Number of attempts exceeded. Account for user {user_id} was locked'
                    ).with_details('user_id', user_id)
                else:
                    raise BadRequestException(
                        context,
                        'WRONG_PASSWORD',
                        'Invalid password'
                    ).with_details('user_id', user_id)

        user_password.fail_count = 0
        user_password.fail_time = None

        self._persistence.update(context, user_password)
        self._activities_connector.log_signin_activity(context, user_id)

        return True

    def change_password(self, context: Optional[IContext], user_id: str, old_password: str, new_password: str) -> None:
        
        self._verify_password(context, new_password, user_id)

        old_password_hashed = self._hash_password(old_password)
        new_password_hashed = self._hash_password(new_password)

        user_password = self._read_user_password(context, user_id)

        if user_password.password != old_password_hashed:
            raise BadRequestException(
                context,
                'WRONG_PASSWORD',
                'Invalid password'
            ).with_details('user_id', user_id)

        if old_password_hashed == new_password_hashed:
            raise BadRequestException(
                context,
                'PASSWORD_NOT_CHANGED',
                'Old and new passwords are identical'
            ).with_details('user_id', user_id)

        user_password = self._add_old_password(user_password, old_password_hashed)
        user_password.password = new_password_hashed
        user_password.rec_code = None
        user_password.rec_expire_time = None
        user_password.locked = False
        user_password.change_time = None

        self._persistence.update(context, user_password)

        self._activities_connector.log_password_changed_activity(context, user_id)
        self._message_distribution_client.sendPasswordChangedEmail(context, user_id)

    def validate_code(self, context: Optional[IContext], user_id: str, code: str) -> bool:
        data = self._read_user_password(context, user_id)
        if data is None:
            return False

        now = datetime.utcnow()
        valid = (
            code == self._magic_code or
            (data.rec_code == code and data.rec_expire_time and data.rec_expire_time > now)
        )
        return valid

    def reset_password(self, context: Optional[IContext], user_id: str, code: str, password: str) -> None:
        self._verify_password(context, password, user_id)

        password_hashed = self._hash_password(password)

        user_password = self._read_user_password(context, user_id)

        if user_password.rec_code != code and code != self._magic_code:
            raise BadRequestException(
                context,
                'WRONG_CODE',
                f'Invalid password recovery code {code}'
            ).with_details('user_id', user_id)

        if (not user_password.rec_expire_time or user_password.rec_expire_time <= datetime.utcnow()) and code != self._magic_code:
            raise BadRequestException(
                context,
                'CODE_EXPIRED',
                f'Password recovery code {code} expired'
            ).with_details('user_id', user_id)

        user_password.password = password_hashed
        user_password.rec_code = None
        user_password.rec_expire_time = None
        user_password.locked = False
        user_password.change_time = None 

        self._persistence.update(context, user_password)

        self._activities_connector.log_password_changed_activity(context, user_id)
        self._message_distribution_client.sendPasswordChangedEmail(context, user_id)

    def recover_password(self, context: Optional[IContext], user_id: str) -> None:
        user_password = self._persistence.get_one_by_id(context, user_id)
        if user_password is None:
            raise NotFoundException(context, 'USER_NOT_FOUND', f'User {user_id} was not found')

        current_time = datetime.utcnow()
        expire_time = current_time + timedelta(milliseconds=self._rec_expire_timeout)

        user_password.rec_code = self._generate_verification_code()
        user_password.rec_expire_time = expire_time

        self._persistence.update(context, user_password)

        self._activities_connector.log_password_recovered_activity(context, user_id)
        self._message_distribution_client.sendRecoverPasswordEmail(context, user_id)
    





# sessions_memory_persistence.py
from typing import Optional
from datetime import datetime
from pip_services4_data.query import FilterParams, PagingParams, DataPage
from pip_services4_components.context import IContext
from pip_services4_observability.log import CompositeLogger
from pip_services4_persistence.persistence import IdentifiableMemoryPersistence
from .ISessionsPersistence import ISessionsPersistence

from ..data.SessionV1 import SessionV1

class SessionsMemoryPersistence(IdentifiableMemoryPersistence, ISessionsPersistence):
    def __init__(self):
        super().__init__()

    def _compose_filter(self, filter_params: FilterParams):
        filter_params = filter_params or FilterParams()

        id = filter_params.get_as_nullable_string('id')
        user_id = filter_params.get_as_nullable_string('user_id')
        active = filter_params.get_as_nullable_boolean('active')
        from_time = filter_params.get_as_nullable_datetime('from_time')
        to_time = filter_params.get_as_nullable_datetime('to_time')

        def predicate(item: SessionV1):
            if id and item.id != id:
                return False
            if user_id and item.user_id != user_id:
                return False
            if active is not None and item.active != active:
                return False
            if from_time and item.request_time < from_time:
                return False
            if to_time and item.request_time >= to_time:
                return False
            return True

        return predicate

    def get_page_by_filter(self, context: Optional[IContext], filter_params: FilterParams, paging: PagingParams) -> DataPage:
        return super().get_page_by_filter(context, self._compose_filter(filter_params), paging)

    def create(self, context: Optional[IContext], item: SessionV1) -> SessionV1:
        item.open_time = datetime.utcnow()
        item.request_time = item.open_time
        return super().create(context, item)

    def update(self, context: Optional[IContext], item: SessionV1) -> SessionV1:
        item.request_time = datetime.utcnow()
        return super().update(context, item)

    def close_expired(self, context: Optional[IContext], request_time: datetime):
        now = datetime.utcnow()
        count = 0
        for item in self._items:
            if item.active and item.request_time < request_time:
                item.active = False
                item.close_time = now
                item.request_time = now
                item.user = None
                item.data = None
                count += 1

        if count > 0:
            self._logger.debug(context, f'Closed {count} expired sessions')



# -*- coding: utf-8 -*-
import json
from typing import Optional

import bottle
from pip_services4_commons.convert import TypeCode
from pip_services4_components.refer import Descriptor, IReferences
from pip_services4_data.validate import ObjectSchema
from pip_services4_http.controller import RestOperations, RestController
from pip_services4_components.context import Context

from eic_aichat_users.accounts.data import AccountV1, AccountV1Schema
from eic_aichat_users.accounts.logic.IAccountsService import IAccountsService
from eic_aichat_users.passwords.logic.IPasswordsService import IPasswordsService
from eic_aichat_users.sessions.logic.ISessionsService import ISessionsService
from eic_aichat_users.roles.logic.IRolesService import IRolesService


class AccountsOperations(RestOperations):
    def __init__(self):
        super().__init__()
        self._accounts_service: IAccountsService = None
        self._passwords_service: IPasswordsService = None
        self._sessions_service: ISessionsService = None
        self._roles_service: IRolesService = None
        self._dependency_resolver.put("accounts-service", Descriptor('aichatusers-accounts', 'service', '*', '*', '1.0'))
        self._dependency_resolver.put("passwords-service", Descriptor('aichatusers-passwords', 'service', '*', '*', '1.0'))
        self._dependency_resolver.put("sessions-service", Descriptor("aichatusers-sessions", "service", "*", "*", "1.0"))
        self._dependency_resolver.put("roles-service", Descriptor('aichatusers-roles', 'service', '*', '*', '1.0'))

    def configure(self, config):
        super().configure(config)

    def set_references(self, references: IReferences):
        super().set_references(references)
        self._accounts_service = self._dependency_resolver.get_one_required('accounts-service')
        self._passwords_service = self._dependency_resolver.get_one_required('passwords-service') 
        self._sessions_service = self._dependency_resolver.get_one_required("sessions-service")
        self._roles_service = self._dependency_resolver.get_one_required("roles-service")

    def get_accounts(self):
        context = Context.from_trace_id(self._get_trace_id())
        filter_params = self._get_filter_params()
        paging_params = self._get_paging_params()
        try:
            res = self._accounts_service.get_accounts(context, filter_params, paging_params)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def get_account_by_id(self, id):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._accounts_service.get_account_by_id(context, id)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def get_account_by_login(self, login):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._accounts_service.get_account_by_login(context, login)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def get_account_by_id_or_login(self, id_or_login):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._accounts_service.get_account_by_id_or_login(context, id_or_login)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def create_account(self):
        context = Context.from_trace_id(self._get_trace_id())
        data = bottle.request.json
        account = data if isinstance(data, dict) else json.loads(data)
        account = None if not account else AccountV1(**account)
        try:
            # Create account
            res = self._accounts_service.create_account(context, account)

            # Create password for the account
            password = data.get("password")
            if password is not None:
                self._passwords_service.set_password(context, res.id, password)
            else:
                pwd = self._passwords_service.set_temp_password(context, res.id)
                password = pwd

            if res is not None:
                result = res.to_dict() if hasattr(res, "to_dict") else res.__dict__.copy()
                result["password"] = password
                return self._send_result(result)

            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    # TODO: check
    def update_account(self, user_id):
        context = Context.from_trace_id(self._get_trace_id())
        
        # Retrieving and deserializing the request body
        data = bottle.request.json
        account_data = data if isinstance(data, dict) else json.loads(data or '{}')

        # Assign an ID to the account object
        account_data['id'] = user_id
        account = AccountV1(**account_data)

        try:
            # Account Update
            new_account: Optional[AccountV1] = self._accounts_service.update_account(context, account)

            # Updating session data, if applicable
            session_id = data.get("session_id")
            user_data = data.get("user")  

            if new_account and session_id and user_data and new_account.id == user_data.get('id'):
                # Merging new account data with the current user
                updated_user = {**user_data, **new_account.to_dict()}
                self._sessions_service.update_session_user(context, session_id, updated_user)

            return self._send_result(new_account if new_account else None, status=200 if new_account else 204)

        except Exception as err:
            return self._send_error(err)

    def delete_account_by_id(self, id):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._accounts_service.delete_account_by_id(context, id)
            roles = self._roles_service.get_roles_by_id(context, id)
            self._roles_service.revoke_roles(context, id, roles)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def drop_account_by_id(self, id):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._accounts_service.drop_account_by_id(context, id)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def register_routes(self, controller: RestController):
        controller.register_route('get', '/users', None, self.get_accounts)

        controller.register_route('get', '/users/<id>', ObjectSchema(True)
                                  .with_optional_property("id", TypeCode.String),
                                  self.get_account_by_id)

        controller.register_route('get', '/users/by_login/<login>', ObjectSchema(True)
                                  .with_optional_property("login", TypeCode.String),
                                  self.get_account_by_login)

        controller.register_route('get', '/users/by_id_or_login/<id_or_login>', ObjectSchema(True)
                                  .with_optional_property("id_or_login", TypeCode.String),
                                  self.get_account_by_id_or_login)

        controller.register_route('post', '/users', ObjectSchema(True)
                                  .with_required_property("body", AccountV1Schema()),
                                  self.create_account)

        controller.register_route('put', '/users/<user_id>', ObjectSchema(True)
                                  .with_required_property("user_id", TypeCode.String)
                                  .with_required_property("body", AccountV1Schema()),
                                  self.update_account)

        controller.register_route('delete', '/users/<id>', ObjectSchema(True)
                                  .with_required_property("id", TypeCode.String),
                                  self.delete_account_by_id)

        controller.register_route('delete', '/users/<id>/drop', ObjectSchema(True)
                                  .with_required_property("id", TypeCode.String),
                                  self.drop_account_by_id)



# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Optional

from pip_services4_components.context import IContext

from ..data.version1.UserPasswordV1 import UserPasswordV1


class IPasswordsPersistence(ABC):

    @abstractmethod
    def get_one_by_id(self, context: Optional[IContext], id: str) -> Optional[UserPasswordV1]:
        pass

    @abstractmethod
    def create(self, context: Optional[IContext], item: UserPasswordV1) -> UserPasswordV1:
        pass

    @abstractmethod
    def update(self, context: Optional[IContext], item: UserPasswordV1) -> UserPasswordV1:
        pass

    @abstractmethod
    def delete_by_id(self, context: Optional[IContext], id: str) -> Optional[UserPasswordV1]:
        pass



# -*- coding: utf-8 -*-

__all__ = [
    'IPasswordsPersistence', 'PasswordsMemoryPersistence', 'PasswordsMongoDbPersistence'
]

from .IPasswordsPersistence import IPasswordsPersistence
from .PasswordsMemoryPersistence import PasswordsMemoryPersistence
from .PasswordsMongoDbPersistence import PasswordsMongoDbPersistence



# -*- coding: utf-8 -*-

__all__ = [
    'ISessionsPersistence', 'SessionsMemoryPersistence', 'SessionsMongoDbPersistence'
]

from .ISessionsPersistence import ISessionsPersistence
from .SessionsMemoryPersistence import SessionsMemoryPersistence
from .SessionsMongoDbPersistence import SessionsMongoDbPersistence



# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Optional
from pip_services4_components.context import IContext
from pip_services4_components.config import ConfigParams
from pip_services4_data.query import FilterParams, PagingParams, DataPage

from ..data.SettingsSectionV1 import SettingsSectionV1


class ISettingsService(ABC):

    @abstractmethod
    def get_section_ids(self, context: Optional[IContext], filter_params: FilterParams,
                        paging: PagingParams) -> DataPage[str]:
        pass

    @abstractmethod
    def get_sections(self, context: Optional[IContext], filter_params: FilterParams,
                     paging: PagingParams) -> DataPage:
        pass

    @abstractmethod
    def get_section_by_id(self, context: Optional[IContext], id: str) -> ConfigParams:
        pass

    @abstractmethod
    def set_section(self, context: Optional[IContext], id: str,
                    parameters: ConfigParams) -> ConfigParams:
        pass

    @abstractmethod
    def modify_section(self, context: Optional[IContext], id: str,
                       update_params: ConfigParams,
                       increment_params: ConfigParams) -> ConfigParams:
        pass



# -*- coding: utf-8 -*-
from typing import Optional
from datetime import datetime

from pip_services4_data.data import IStringIdentifiable


class UserPasswordInfoV1(IStringIdentifiable):
    def __init__(
        self,
        id: Optional[str] = None,
        change_time: Optional[datetime] = None,
        locked: bool = False,
        lock_time: Optional[datetime] = None
    ):
        self.id: Optional[str] = id
        self.change_time: Optional[datetime] = change_time
        self.locked: bool = locked
        self.lock_time: Optional[datetime] = lock_time
        

    def to_dict(self):
        return self.__dict__



# -*- coding: utf-8 -*-
from typing import Optional

from pip_services4_components.context import IContext
from pip_services4_components.config import ConfigParams
from pip_services4_data.query import FilterParams, PagingParams, DataPage
from pip_services4_mongodb.persistence import IdentifiableMongoDbPersistence

from ..data import PartyActivityV1, ReferenceV1


class ActivitiesMongoDbPersistence(IdentifiableMongoDbPersistence):
    """
    MongoDB persistence for party activities.
    """

    def _convert_reference_from_public(self, reference: ReferenceV1) -> dict:
        if reference is None:
            return None
        return reference.__dict__

    def _convert_reference_to_public(self, reference: dict) -> ReferenceV1:
        if reference is None:
            return None
        return ReferenceV1(
            id=reference.get('id'),
            type=reference.get('type'),
            name=reference.get('name')
        )

    def _convert_from_public(self, item: PartyActivityV1) -> any:
        # return {
        #     '_id': item.id,
        #     'time': item.time,
        #     'party': self._convert_reference_from_public(item.party),
        #     'type': item.type,
        #     'ref_item': self._convert_reference_from_public(item.ref_item),
        #     'ref_parents': [self._convert_reference_from_public(ref_parent) for ref_parent in item.ref_parents],
        #     'ref_party': self._convert_reference_from_public(item.ref_party),
        #     'details': item.details
        # }
    
        result = item.__dict__.copy()
        result['_id'] = item.id

        if item.party:
            result['party'] = self._convert_reference_from_public(item.party)
        if item.ref_item:
            result['ref_item'] = self._convert_reference_from_public(item.ref_item)
        if item.ref_party:
            result['ref_party'] = self._convert_reference_from_public(item.ref_party)

        if item.ref_parents:
            result['ref_parents'] = [self._convert_reference_from_public(ref_parent) for ref_parent in item.ref_parents]

        return result

    def _convert_to_public(self, value: any) -> any:
        if value is None:
            return None
        
        return PartyActivityV1(
            id=value.get('_id'),
            time=value.get('time'),
            party=self._convert_reference_to_public(value.get('party')),
            type=value.get('type'),
            ref_item=self._convert_reference_to_public(value.get('ref_item')),
            ref_parents=[self._convert_reference_to_public(ref_parent) for ref_parent in value.get('ref_parents')],
            ref_party=self._convert_reference_to_public(value.get('ref_party')),
            details=value.get('details', {})
        )

    def __init__(self):
        """
        Creates a new instance of the persistence
        """
        super().__init__('activities')
        self._max_page_size = 1000

    def _compose_filter(self, filter_params: FilterParams):
        filter_params = filter_params or FilterParams()
        
        criteria = {}
        
        id = filter_params.get_as_nullable_string('id')
        if id is not None:
            criteria['_id'] = id
        
        ids = filter_params.get_as_nullable_string('ids')
        if ids is not None:
            criteria['ids'] = ids
            
        party_id = filter_params.get_as_nullable_string('party_id')
        if party_id is not None:
            criteria['party.id'] = party_id
            
        ref_item_id = filter_params.get_as_nullable_string('ref_item_id')
        if ref_item_id is not None:
            criteria['ref_item.id'] = ref_item_id
            
        ref_parent_id = filter_params.get_as_nullable_string('ref_parent_id')
        if ref_parent_id is not None:
            criteria['ref_parents.id'] = ref_parent_id
            
        ref_party_id = filter_params.get_as_nullable_string('ref_party_id')
        if ref_party_id is not None:
            criteria['ref_party.id'] = ref_party_id
            
        type = filter_params.get_as_nullable_string('type')
        if type is not None:
            criteria['type'] = type
            
        return criteria

    def get_page_by_filter(self, context: IContext, filter_params: FilterParams, paging_params: PagingParams) -> DataPage:
        """
        Gets a page of party activities by specified filter
        
        Args:
            context: Operation context
            filter_params: Filter parameters
            paging_params: Paging parameters
            
        Returns:
            DataPage with retrieved party activities
        """
        return super().get_page_by_filter(context, self._compose_filter(filter_params), paging_params, None, None)

    def delete_by_filter(self, context: IContext, filter_params: FilterParams) -> None:
        """
        Deletes party activities by filter
        
        Args:
            context: Operation context
            filter_params: Filter parameters
        """

        criteria = self._compose_filter(filter_params)
        return super().delete_by_ids(context, criteria.get('ids'))
    
    def create(self, context: IContext, item: PartyActivityV1) -> PartyActivityV1:
        if item is None:
            return

        new_item = self._convert_from_public(item)

        result = self._collection.insert_one(new_item)
        item = self._collection.find_one({'_id': result.inserted_id})

        item = self._convert_to_public(item)
        return item
        


# -*- coding: utf-8 -*-
from pip_services4_commons.convert import TypeCode
from pip_services4_components.refer import Descriptor, IReferences
from pip_services4_data.validate import ObjectSchema
from pip_services4_http.controller import RestOperations, RestController
from pip_services4_components.context import Context
import bottle

from eic_aichat_users.passwords.logic.IPasswordsService import IPasswordsService
from eic_aichat_users.accounts.logic.IAccountsService import IAccountsService

class PasswordsOperations(RestOperations):
    def __init__(self):
        super().__init__()
        self._passwords_service: IPasswordsService = None
        self._accounts_service: IAccountsService = None
        self._dependency_resolver.put("passwords-service", Descriptor('aichatusers-passwords', 'service', '*', '*', '1.0'))
        self._dependency_resolver.put("accounts-service", Descriptor('aichatusers-accounts', 'service', '*', '*', '1.0'))

    def configure(self, config):
        super().configure(config)

    def set_references(self, references: IReferences):
        super().set_references(references)
        self._passwords_service = self._dependency_resolver.get_one_required('passwords-service') 
        self._accounts_service = self._dependency_resolver.get_one_required('accounts-service')
        
    def change_password(self, user_id: str):
        context = Context.from_trace_id(self._get_trace_id())
        data = bottle.request.json or {}
        try:
            res = self._passwords_service.change_password(
                context, 
                user_id, 
                data.get("old_password"), 
                data.get("new_password")
                )
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)    
        
    def reset_password(self):
        context = Context.from_trace_id(self._get_trace_id())
        data = bottle.request.json or {}
        try:
            account = self._accounts_service.get_account_by_id_or_login(context, data.get("login"))
            res = self._passwords_service.reset_password(
                context, 
                account.id, 
                data.get("code"), 
                data.get("password")
                )
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)  
        
    def recover_password(self):
        context = Context.from_trace_id(self._get_trace_id())
        data = bottle.request.json or {}
        try:
            account = self._accounts_service.get_account_by_id_or_login(context, data.get("login"))
            res = self._passwords_service.recover_password(context, account.id)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)   
        
    def register_routes(self, controller: RestController):

        controller.register_route('post', 'users/<user_id>/passwords/change', ObjectSchema(True)
                                  .with_required_property("user_id", TypeCode.String)
                                  .with_required_property("body", TypeCode.Map),
                                  self.change_password)

        controller.register_route('post', 'users/passwords/reset', ObjectSchema(True)
                                  .with_required_property("body", TypeCode.Map),
                                  self.reset_password)
        
        controller.register_route('post', 'users/passwords/recover', ObjectSchema(True)
                                  .with_required_property("body", TypeCode.Map),
                                  self.recover_password)


        